@@ -592,7 +592,6 @@ public class COSDictionary extends COSBase
 
     /**
      * Convenience method that will get the dictionary object that is expected to be a name and convert it to a string.
-     * Null is returned if the entry does not exist in the dictionary.
      *
      * @param key The key to the item in the dictionary.
      * @param defaultValue The value to return if the dictionary item is null.
@@ -605,7 +604,6 @@ public class COSDictionary extends COSBase
 
     /**
      * Convenience method that will get the dictionary object that is expected to be a name and convert it to a string.
-     * Null is returned if the entry does not exist in the dictionary.
      *
      * @param key The key to the item in the dictionary.
      * @param defaultValue The value to return if the dictionary item is null.
@@ -643,7 +641,6 @@ public class COSDictionary extends COSBase
 
     /**
      * Convenience method that will get the dictionary object that is expected to be a name and convert it to a string.
-     * Null is returned if the entry does not exist in the dictionary.
      *
      * @param key The key to the item in the dictionary.
      * @param defaultValue The default value to return.
@@ -656,7 +653,6 @@ public class COSDictionary extends COSBase
 
     /**
      * Convenience method that will get the dictionary object that is expected to be a name and convert it to a string.
-     * Null is returned if the entry does not exist in the dictionary.
      *
      * @param key The key to the item in the dictionary.
      * @param defaultValue The default value to return.
@@ -709,7 +705,6 @@ public class COSDictionary extends COSBase
 
     /**
      * Convenience method that will get the dictionary object that is expected to be a name and convert it to a string.
-     * Null is returned if the entry does not exist in the dictionary.
      *
      * @param embedded The embedded dictionary.
      * @param key The key to the item in the dictionary.
@@ -1167,6 +1162,26 @@ public class COSDictionary extends COSBase
         return getItem(COSName.getPDFName(key));
     }
 
+    /**
+     * This is a special case of getItem that takes multiple keys, it will handle the situation
+     * where multiple keys could get the same value, ie if either CS or ColorSpace is used to get
+     * the colorspace. This will get an object from this dictionary.
+     *
+     * @param firstKey The first key to try.
+     * @param secondKey The second key to try.
+     *
+     * @return The object that matches the key.
+     */
+    public COSBase getItem(COSName firstKey, COSName secondKey)
+    {
+        COSBase retval = getItem(firstKey);
+        if (retval == null && secondKey != null)
+        {
+            retval = getItem(secondKey);
+        }
+        return retval;
+    }
+
     /**
      * @return names of the entries in this dictionary. The returned set is in the order the entries were added to the
      * dictionary.
