@@ -40,6 +40,7 @@ import java.util.Optional;
 import java.util.Set;
 
 import org.sejda.io.CountingWritableByteChannel;
+import org.sejda.io.SeekableSources;
 import org.sejda.sambox.cos.COSArray;
 import org.sejda.sambox.cos.COSBase;
 import org.sejda.sambox.cos.COSDictionary;
@@ -52,13 +53,14 @@ import org.sejda.sambox.cos.DirectCOSObject;
 import org.sejda.sambox.encryption.EncryptionContext;
 import org.sejda.sambox.encryption.MessageDigests;
 import org.sejda.sambox.encryption.StandardSecurity;
+import org.sejda.sambox.input.PDFParser;
 import org.sejda.sambox.output.PDDocumentWriter;
 import org.sejda.sambox.output.WriteOption;
 import org.sejda.sambox.pdmodel.common.PDStream;
 import org.sejda.sambox.pdmodel.encryption.AccessPermission;
 import org.sejda.sambox.pdmodel.encryption.PDEncryption;
 import org.sejda.sambox.pdmodel.encryption.SecurityHandler;
-import org.sejda.sambox.pdmodel.font.PDFont;
+import org.sejda.sambox.pdmodel.font.Subsettable;
 import org.sejda.sambox.pdmodel.graphics.color.PDDeviceRGB;
 import org.sejda.sambox.util.Version;
 import org.sejda.util.IOUtils;
@@ -108,7 +110,7 @@ public class PDDocument implements Closeable
     private ResourceCache resourceCache = new DefaultResourceCache();
 
     // fonts to subset before saving
-    private final Set<PDFont> fontsToSubset = new HashSet<>();
+    private final Set<Subsettable> fontsToSubset = new HashSet<>();
 
     public PDDocument()
     {
@@ -286,7 +288,7 @@ public class PDDocument implements Closeable
     /**
      * @return the list of fonts which will be subset before the document is saved.
      */
-    Set<PDFont> getFontsToSubset()
+    public Set<Subsettable> getFontsToSubset()
     {
         return fontsToSubset;
     }
@@ -569,7 +571,7 @@ public class PDDocument implements Closeable
         requireOpen();
         getDocumentInformation().setProducer("SAMBox " + Version.getVersion() + " (www.sejda.org)");
         getDocumentInformation().setModificationDate(Calendar.getInstance());
-        for (PDFont font : fontsToSubset)
+        for (Subsettable font : fontsToSubset)
         {
             font.subset();
         }
@@ -637,4 +639,9 @@ public class PDDocument implements Closeable
         return resourceCache;
     }
 
+    // bridge to pdfbox style api, used in tests
+    public static PDDocument load(File file) throws IOException {
+        return PDFParser.parse(SeekableSources.seekableSourceFrom(file));
+    }
+
 }
