@@ -19,7 +19,6 @@ package org.sejda.sambox.pdmodel.font;
 import static java.util.Objects.nonNull;
 
 import java.io.ByteArrayInputStream;
-import java.io.ByteArrayOutputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.util.Collections;
@@ -32,6 +31,7 @@ import org.apache.commons.io.IOUtils;
 import org.apache.fontbox.afm.FontMetrics;
 import org.apache.fontbox.cmap.CMap;
 import org.apache.fontbox.util.BoundingBox;
+import org.sejda.io.FastByteArrayOutputStream;
 import org.sejda.sambox.cos.COSArray;
 import org.sejda.sambox.cos.COSArrayList;
 import org.sejda.sambox.cos.COSBase;
@@ -51,7 +51,7 @@ import org.slf4j.LoggerFactory;
  * 
  * @author Ben Litchfield
  */
-public abstract class PDFont implements COSObjectable, PDFontLike
+public abstract class PDFont implements COSObjectable, PDFontLike, Subsettable
 {
     private static final Logger LOG = LoggerFactory.getLogger(PDFont.class);
     protected static final Matrix DEFAULT_FONT_MATRIX = new Matrix(0.001f, 0, 0, 0.001f, 0, 0);
@@ -312,8 +312,9 @@ public abstract class PDFont implements COSObjectable, PDFontLike
      */
     public final byte[] encode(String text) throws IOException
     {
-        ByteArrayOutputStream out = new ByteArrayOutputStream();
-        for (int offset = 0; offset < text.length();)
+        FastByteArrayOutputStream out = new FastByteArrayOutputStream();
+        int offset = 0;
+        while (offset < text.length())
         {
             int codePoint = text.codePointAt(offset);
 
@@ -326,6 +327,40 @@ public abstract class PDFont implements COSObjectable, PDFontLike
         return out.toByteArray();
     }
 
+    /**
+     * Similar to encode() but handles leniently cases where fonts don't have a glyph by assuming the identity mapping
+     */
+    public final byte[] encodeLeniently(String text) throws IOException
+    {
+        FastByteArrayOutputStream out = new FastByteArrayOutputStream();
+        for (int offset = 0; offset < text.length();)
+        {
+            int codePoint = text.codePointAt(offset);
+
+            // multi-byte encoding with 1 to 4 bytes
+            byte[] bytes;
+            try
+            {
+                bytes = encode(codePoint);
+            }
+            catch (IllegalArgumentException e)
+            {
+                if (e.getMessage().contains("No glyph"))
+                {
+                    bytes = new byte[] { (byte) codePoint };
+                }
+                else
+                {
+                    throw e;
+                }
+            }
+            out.write(bytes);
+
+            offset += Character.charCount(codePoint);
+        }
+        return out.toByteArray();
+    }
+
     /**
      * Encodes the given Unicode code point for use in a PDF content stream. Content streams use a multi-byte encoding
      * with 1 to 4 bytes.
@@ -361,6 +396,27 @@ public abstract class PDFont implements COSObjectable, PDFontLike
         return width;
     }
 
+    /**
+     * Similar to getStringWidth() but handles leniently fonts where glyphs are missing, assuming the identity mapping
+     * of glyphs
+     *
+     * Uses encodeLeniently() instead of encode()
+     */
+    public float getStringWidthLeniently(String text) throws IOException
+    {
+        byte[] bytes = encodeLeniently(text);
+        ByteArrayInputStream in = new ByteArrayInputStream(bytes);
+
+        float width = 0;
+        while (in.available() > 0)
+        {
+            int code = readCode(in);
+            width += getWidth(code);
+        }
+
+        return width;
+    }
+
     /**
      * This will get the average font width for all characters.
      *
@@ -575,25 +631,6 @@ public abstract class PDFont implements COSObjectable, PDFontLike
         return Standard14Fonts.containsName(getName());
     }
 
-    /**
-     * Adds the given Unicode point to the subset.
-     * 
-     * @param codePoint Unicode code point
-     */
-    public abstract void addToSubset(int codePoint);
-
-    /**
-     * Replaces this font with a subset containing only the given Unicode characters.
-     *
-     * @throws IOException if the subset could not be written
-     */
-    public abstract void subset() throws IOException;
-
-    /**
-     * Returns true if this font will be subset when embedded.
-     */
-    public abstract boolean willBeSubset();
-
     @Override
     public abstract boolean isDamaged();
 
