@@ -19,12 +19,15 @@ package org.sejda.sambox.pdmodel.graphics.color;
 import static org.sejda.util.RequireUtils.requireIOCondition;
 
 import java.awt.Color;
+import java.awt.Transparency;
 import java.awt.color.CMMException;
 import java.awt.color.ColorSpace;
 import java.awt.color.ICC_ColorSpace;
 import java.awt.color.ICC_Profile;
 import java.awt.color.ProfileDataException;
 import java.awt.image.BufferedImage;
+import java.awt.image.ComponentColorModel;
+import java.awt.image.DataBuffer;
 import java.awt.image.WritableRaster;
 import java.io.IOException;
 import java.io.InputStream;
@@ -131,6 +134,7 @@ public final class PDICCBased extends PDCIEBasedColorSpace
                 }
                 else
                 {
+                    profile = ensureDisplayProfile(profile);
                     awtColorSpace = new ICC_ColorSpace(profile);
                     iccProfile = profile;
                 }
@@ -149,29 +153,30 @@ public final class PDICCBased extends PDCIEBasedColorSpace
                 awtColorSpace.toRGB(new float[awtColorSpace.getNumComponents()]);
                 // this one triggers an exception for PDFBOX-3549 with KCMS
                 new Color(awtColorSpace, new float[getNumberOfComponents()], 1f);
+                // PDFBOX-4015: this one triggers "CMMException: LCMS error 13" with LCMS
+                new ComponentColorModel(awtColorSpace, false, false, Transparency.OPAQUE,
+                        DataBuffer.TYPE_BYTE);
             }
         }
-        catch (RuntimeException e)
+        catch (ProfileDataException e)
         {
-            if (e instanceof ProfileDataException || e instanceof CMMException
-                    || e instanceof IllegalArgumentException
-                    || e instanceof ArrayIndexOutOfBoundsException)
-            {
-                // fall back to alternateColorSpace color space
-                awtColorSpace = null;
-                alternateColorSpace = getAlternateColorSpace();
-                if (alternateColorSpace.equals(PDDeviceRGB.INSTANCE))
-                {
-                    isRGB = true;
-                }
-                LOG.warn("Can't read embedded ICC profile (" + e.getLocalizedMessage()
-                        + "), using alternate color space: " + alternateColorSpace.getName());
-                initialColor = alternateColorSpace.getInitialColor();
-            }
-            else
-            {
-                throw e;
-            }
+            fallbackToAlternateColorSpace(e);
+        }
+        catch (CMMException e)
+        {
+            fallbackToAlternateColorSpace(e);
+        }
+        catch (IllegalArgumentException e)
+        {
+            fallbackToAlternateColorSpace(e);
+        }
+        catch (ArrayIndexOutOfBoundsException e)
+        {
+            fallbackToAlternateColorSpace(e);
+        }
+        catch (IOException e)
+        {
+            fallbackToAlternateColorSpace(e);
         }
         finally
         {
@@ -179,6 +184,19 @@ public final class PDICCBased extends PDCIEBasedColorSpace
         }
     }
 
+    private void fallbackToAlternateColorSpace(Exception e) throws IOException
+    {
+        awtColorSpace = null;
+        alternateColorSpace = getAlternateColorSpace();
+        if (alternateColorSpace.equals(PDDeviceRGB.INSTANCE))
+        {
+            isRGB = true;
+        }
+        LOG.warn("Can't read embedded ICC profile (" + e.getLocalizedMessage() +
+                "), using alternate color space: " + alternateColorSpace.getName());
+        initialColor = alternateColorSpace.getInitialColor();
+    }
+
     /**
      * Returns true if the given profile is represents sRGB.
      */
@@ -190,6 +208,33 @@ public final class PDICCBased extends PDCIEBasedColorSpace
         return deviceModel.equals("sRGB");
     }
 
+    // PDFBOX-4114: fix profile that has the wrong display class,
+    // as done by Harald Kuhr in twelvemonkeys JPEGImageReader.ensureDisplayProfile()
+    private static ICC_Profile ensureDisplayProfile(ICC_Profile profile)
+    {
+        if (profile.getProfileClass() != ICC_Profile.CLASS_DISPLAY)
+        {
+            byte[] profileData = profile.getData(); // Need to clone entire profile, due to a OpenJDK bug
+
+            if (profileData[ICC_Profile.icHdrRenderingIntent] == ICC_Profile.icPerceptual)
+            {
+                LOG.warn("ICC profile is Perceptual, ignoring, treating as Display class");
+                intToBigEndian(ICC_Profile.icSigDisplayClass, profileData, ICC_Profile.icHdrDeviceClass);
+                return ICC_Profile.getInstance(profileData);
+            }
+        }
+        return profile;
+    }
+
+    private static void intToBigEndian(int value, byte[] array, int index)
+    {
+        array[index] = (byte) (value >> 24);
+        array[index + 1] = (byte) (value >> 16);
+        array[index + 2] = (byte) (value >> 8);
+        array[index + 3] = (byte) (value);
+    }
+
+
     @Override
     public float[] toRGB(float[] value) throws IOException
     {
@@ -199,12 +244,25 @@ public final class PDICCBased extends PDCIEBasedColorSpace
         }
         if (awtColorSpace != null)
         {
+            // PDFBOX-2142: clamp bad values
             // WARNING: toRGB is very slow when used with LUT-based ICC profiles
-            return awtColorSpace.toRGB(value);
+            return awtColorSpace.toRGB(clampColors(awtColorSpace, value));
         }
         return alternateColorSpace.toRGB(value);
     }
 
+    private float[] clampColors(ICC_ColorSpace cs, float[] value)
+    {
+        float[] result = new float[value.length];
+        for (int i = 0; i < value.length; ++i)
+        {
+            float minValue = cs.getMinValue(i);
+            float maxValue = cs.getMaxValue(i);
+            result[i] = value[i] < minValue ? minValue : (value[i] > maxValue ? maxValue : value[i]);
+        }
+        return result;
+    }
+
     @Override
     public BufferedImage toRGBImage(WritableRaster raster) throws IOException
     {
