@@ -27,7 +27,9 @@ import java.io.File;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
+import java.lang.ref.SoftReference;
 import java.nio.ByteBuffer;
+import java.util.List;
 
 import javax.imageio.ImageIO;
 
@@ -36,6 +38,7 @@ import org.sejda.sambox.cos.COSArray;
 import org.sejda.sambox.cos.COSBase;
 import org.sejda.sambox.cos.COSName;
 import org.sejda.sambox.cos.COSStream;
+import org.sejda.sambox.filter.DecodeResult;
 import org.sejda.sambox.pdmodel.PDResources;
 import org.sejda.sambox.pdmodel.common.PDMetadata;
 import org.sejda.sambox.pdmodel.common.PDStream;
@@ -44,6 +47,8 @@ import org.sejda.sambox.pdmodel.graphics.color.PDColorSpace;
 import org.sejda.sambox.pdmodel.graphics.color.PDDeviceGray;
 import org.sejda.sambox.util.filetypedetector.FileType;
 import org.sejda.sambox.util.filetypedetector.FileTypeDetector;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
 /**
  * An Image XObject.
@@ -53,7 +58,10 @@ import org.sejda.sambox.util.filetypedetector.FileTypeDetector;
  */
 public final class PDImageXObject extends PDXObject implements PDImage
 {
-    private BufferedImage cachedImage;
+
+    private static final Logger LOG = LoggerFactory.getLogger(PDImageXObject.class);
+
+    private SoftReference<BufferedImage> cachedImage;
     private PDColorSpace colorSpace;
     private PDResources resources; // current resource dictionary (has color spaces)
 
@@ -74,7 +82,6 @@ public final class PDImageXObject extends PDXObject implements PDImage
     /**
      * Creates an Image XObject in the given document.
      * 
-     * @param document the current document
      * @throws java.io.IOException if there is an error creating the XObject.
      */
     public PDImageXObject() throws IOException
@@ -129,9 +136,15 @@ public final class PDImageXObject extends PDXObject implements PDImage
     public PDImageXObject(PDStream stream, PDResources resources) throws IOException
     {
         super(stream, COSName.IMAGE);
-        stream.getCOSObject().addAll(stream.getCOSObject().getDecodeResult().getParameters());
         this.resources = resources;
-        this.colorSpace = stream.getCOSObject().getDecodeResult().getJPXColorSpace();
+        List<COSName> filters = stream.getFilters();
+        if (filters != null && !filters.isEmpty()
+                && COSName.JPX_DECODE.equals(filters.get(filters.size() - 1)))
+        {
+            DecodeResult decodeResult = stream.getCOSObject().getDecodeResult();
+            stream.getCOSObject().addAll(decodeResult.getParameters());
+            this.colorSpace = decodeResult.getJPXColorSpace();
+        }
     }
 
     public static PDImageXObject createFromFile(String imagePath) throws IOException
@@ -151,11 +164,21 @@ public final class PDImageXObject extends PDXObject implements PDImage
         }
         if (fileType.equals(FileType.TIFF))
         {
-            return CCITTFactory.createFromFile(file);
+            try
+            {
+                return CCITTFactory.createFromFile(file);
+            }
+            catch (IOException ex)
+            {
+                LOG.warn("Reading as TIFF failed, setting fileType to PNG", ex);
+                // Plan B: try reading with ImageIO
+                // common exception:
+                // First image in tiff is not CCITT T4 or T6 compressed
+            }
         }
         // last resort, let's see if ImageIO can read it
         BufferedImage image = ImageIO.read(file);
-        requireNotNullArg(image, "Image type not supported " + file.getName());
+        requireNotNullArg(image, "Image type " + fileType + " not supported " + file.getName());
         return LosslessFactory.createFromImage(image);
     }
 
@@ -212,7 +235,11 @@ public final class PDImageXObject extends PDXObject implements PDImage
     {
         if (cachedImage != null)
         {
-            return cachedImage;
+            BufferedImage cached = cachedImage.get();
+            if (cached != null)
+            {
+                return cached;
+            }
         }
 
         // get image as RGB
@@ -234,7 +261,7 @@ public final class PDImageXObject extends PDXObject implements PDImage
             }
         }
 
-        cachedImage = image;
+        cachedImage = new SoftReference<>(image);
         return image;
     }
 
@@ -340,9 +367,12 @@ public final class PDImageXObject extends PDXObject implements PDImage
     {
         BufferedImage image2 = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
         Graphics2D g = image2.createGraphics();
-        g.setRenderingHint(RenderingHints.KEY_INTERPOLATION,
-                RenderingHints.VALUE_INTERPOLATION_BICUBIC);
-        g.setRenderingHint(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY);
+        if (getInterpolate())
+        {
+            g.setRenderingHint(RenderingHints.KEY_INTERPOLATION,
+                    RenderingHints.VALUE_INTERPOLATION_BICUBIC);
+            g.setRenderingHint(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY);
+        }
         g.drawImage(image, 0, 0, width, height, 0, 0, image.getWidth(), image.getHeight(), null);
         g.dispose();
         return image2;
