@@ -18,13 +18,17 @@ package org.sejda.sambox.pdmodel.interactive.form;
 
 import static java.util.Arrays.asList;
 import static java.util.Objects.nonNull;
+import static java.util.Optional.ofNullable;
 import static org.sejda.io.CountingWritableByteChannel.from;
+import static org.sejda.util.RequireUtils.requireNotNullArg;
 
 import java.awt.geom.AffineTransform;
 import java.awt.geom.Point2D;
 import java.io.IOException;
 import java.util.ArrayList;
+import java.util.HashMap;
 import java.util.List;
+import java.util.Map;
 
 import org.sejda.sambox.contentstream.operator.Operator;
 import org.sejda.sambox.cos.COSName;
@@ -84,6 +88,8 @@ class AppearanceGeneratorHelper
      */
     private static final float DEFAULT_FONT_SIZE = 12;
 
+    private static final int MINIMUM_LINES_TO_FIT_IN_A_MULTILINE_FIELD = 5;
+
     /**
      * The default padding applied by Acrobat to the fields bbox.
      */
@@ -123,6 +129,7 @@ class AppearanceGeneratorHelper
                     && widget.getNormalAppearanceStream().getResources() != null)
             {
                 PDResources widgetResources = widget.getNormalAppearanceStream().getResources();
+                Map<COSName, PDFont> missingFonts = new HashMap<>();
                 for (COSName fontResourceName : widgetResources.getFontNames())
                 {
                     try
@@ -131,7 +138,7 @@ class AppearanceGeneratorHelper
                         {
                             LOG.debug("Adding font resource " + fontResourceName
                                     + " from widget to AcroForm");
-                            acroFormResources.put(fontResourceName,
+                            missingFonts.put(fontResourceName,
                                     widgetResources.getFont(fontResourceName));
                         }
                     }
@@ -140,6 +147,12 @@ class AppearanceGeneratorHelper
                         LOG.warn("Unable to match field level font with AcroForm font");
                     }
                 }
+
+                // add all missing font resources from widget to AcroForm
+                for (COSName key : missingFonts.keySet())
+                {
+                    acroFormResources.put(key, missingFonts.get(key));
+                }
             }
         }
     }
@@ -253,7 +266,15 @@ class AppearanceGeneratorHelper
     {
         COSString da = (COSString) widget.getCOSObject().getDictionaryObject(COSName.DA);
         PDResources dr = field.getAcroForm().getDefaultResources();
-        return new PDDefaultAppearanceString(da, dr);
+        try
+        {
+            return new PDDefaultAppearanceString(da, dr);
+        }
+        catch (IOException ex)
+        {
+            LOG.warn("Failed to process default appearance string for widget {}, will use fallback default appearance", widget);
+            return new PDDefaultAppearanceString();
+        }
     }
 
     private int resolveRotation(PDAnnotationWidget widget)
@@ -415,24 +436,20 @@ class AppearanceGeneratorHelper
         contents.clip();
 
         // get the font
-
         // field's defined appearance font has priority
         // callers might have determined that the default font does not support rendering the field's value
         // so the font was substituted to another one, which has better unicode support
         // see PDVariableText.setAppearanceOverrideFont()
-        PDFont font = field.getAppearanceFont();
+        PDFont font = ofNullable(field.getAppearanceFont())
+                .orElseGet(() -> defaultAppearance.getFont());
 
-        // fallback to default appearance
-        if (font == null)
-        {
-            font = defaultAppearance.getFont();
-        }
+        requireNotNullArg(font, "font is null, check whether /DA entry is incomplete or incorrect");
 
-        // calculate the fontSize (because 0 = autosize)
         float fontSize = defaultAppearance.getFontSize();
 
         if (fontSize == 0)
         {
+            // calculate the fontSize (because 0 = autosize)
             fontSize = calculateFontSize(font, contentRect);
         }
 
@@ -459,7 +476,7 @@ class AppearanceGeneratorHelper
 
         if (field instanceof PDTextField && ((PDTextField) field).isMultiline())
         {
-            y = contentRect.getUpperRightY() - fontBoundingBoxAtSize;
+            y = contentRect.getUpperRightY() - calculateLineHeight(font, fontScaleY);
         }
         else
         {
@@ -493,7 +510,7 @@ class AppearanceGeneratorHelper
         // chars
         if (shallComb())
         {
-            insertGeneratedCombAppearance(contents, appearanceStream, font, fontSize);
+            insertGeneratedCombAppearance(contents, bbox, font, fontSize);
         }
         else if (field instanceof PDListBox)
         {
@@ -508,7 +525,7 @@ class AppearanceGeneratorHelper
             appearanceStyle.setFontSize(fontSize);
 
             // Adobe Acrobat uses the font's bounding box for the leading between the lines
-            appearanceStyle.setLeading(font.getBoundingBox().getHeight() * fontScaleY);
+            appearanceStyle.setLeading(calculateLineHeight(font, fontScaleY));
 
             PlainTextFormatter formatter = new PlainTextFormatter.Builder(contents)
                     .style(appearanceStyle).text(textContent).width(contentRect.getWidth())
@@ -574,13 +591,13 @@ class AppearanceGeneratorHelper
      * Generate the appearance for comb fields.
      * 
      * @param contents the content stream to write to
-     * @param appearanceStream the appearance stream used
+     * @param bbox the bbox used
      * @param font the font to be used
      * @param fontSize the font size to be used
      * @throws IOException
      */
-    private void insertGeneratedCombAppearance(PDPageContentStream contents,
-            PDAppearanceStream appearanceStream, PDFont font, float fontSize) throws IOException
+    private void insertGeneratedCombAppearance(PDPageContentStream contents, PDRectangle bbox,
+            PDFont font, float fontSize) throws IOException
     {
 
         // TODO: Currently the quadding is not taken into account
@@ -589,12 +606,12 @@ class AppearanceGeneratorHelper
         int maxLen = ((PDTextField) field).getMaxLen();
         int numChars = Math.min(value.length(), maxLen);
 
-        PDRectangle paddingEdge = applyPadding(appearanceStream.getBBox(), 1);
+        PDRectangle paddingEdge = applyPadding(bbox, 1);
 
-        float combWidth = appearanceStream.getBBox().getWidth() / maxLen;
+        float combWidth = bbox.getWidth() / maxLen;
         float ascentAtFontSize = font.getFontDescriptor().getAscent() / FONTSCALE * fontSize;
         float baselineOffset = paddingEdge.getLowerLeftY()
-                + (appearanceStream.getBBox().getHeight() - ascentAtFontSize) / 2;
+                + (bbox.getHeight() - ascentAtFontSize) / 2;
 
         float prevCharWidth = 0f;
 
@@ -717,44 +734,77 @@ class AppearanceGeneratorHelper
         }
     }
 
+    private float calculateLineHeight(PDFont font, float fontScaleY) throws IOException
+    {
+        float fontBoundingBoxAtSize = font.getBoundingBox().getHeight() * fontScaleY;
+        float fontCapAtSize = font.getFontDescriptor().getCapHeight() * fontScaleY;
+        float fontDescentAtSize = font.getFontDescriptor().getDescent() * fontScaleY;
+
+        float lineHeight = fontCapAtSize - fontDescentAtSize;
+        if (lineHeight < 0)
+        {
+            lineHeight = fontBoundingBoxAtSize;
+        }
+
+        return lineHeight;
+    }
+
     /**
      * My "not so great" method for calculating the fontsize. It does not work superb, but it handles ok.
      * 
      * @return the calculated font-size
      * @throws IOException If there is an error getting the font information.
      */
-    private float calculateFontSize(PDFont font, PDRectangle contentRect) throws IOException
+    float calculateFontSize(PDFont font, PDRectangle contentRect) throws IOException
     {
-        float fontSize = defaultAppearance.getFontSize();
+        float yScalingFactor = FONTSCALE * font.getFontMatrix().getScaleY();
+        float xScalingFactor = FONTSCALE * font.getFontMatrix().getScaleX();
 
-        // zero is special, it means the text is auto-sized
-        if (fontSize == 0)
+        if (isMultiLine())
         {
-            if (isMultiLine())
+            // Acrobat defaults to 12 for multiline text with size 0
+            // PDFBOX decided to just return that and finish with it
+            // return DEFAULT_FONT_SIZE;
+
+            // SAMBOX specifics below
+            // We calculate a font size that fits at least 5 lines
+            // We detect faux multiline fields (text fields flagged as multiline which have a small height to just fit
+            // one line)
+
+            float lineHeight = calculateLineHeight(font, font.getFontMatrix().getScaleY());
+            float scaledContentHeight = contentRect.getHeight() * yScalingFactor;
+
+            boolean looksLikeFauxMultiline = calculateLineHeight(font, DEFAULT_FONT_SIZE / FONTSCALE) > scaledContentHeight;
+            boolean userTypedMultipleLines = new PlainText(value).getParagraphs().size() > 1;
+
+            if (looksLikeFauxMultiline && !userTypedMultipleLines)
             {
-                // Acrobat defaults to 12 for multiline text with size 0
-                return DEFAULT_FONT_SIZE;
+                // faux multiline detected
+                // because 1 line written with the default font size would not fit the height
+                // just continue to the non multiline part of the algorithm
+
+                LOG.warn("Faux multiline field found: {}", field.getFullyQualifiedName());
             }
-            float yScalingFactor = FONTSCALE * font.getFontMatrix().getScaleY();
-            float xScalingFactor = FONTSCALE * font.getFontMatrix().getScaleX();
-
-            // fit width
-            float width = font.getStringWidth(value) * font.getFontMatrix().getScaleX();
-            float widthBasedFontSize = contentRect.getWidth() / width * xScalingFactor;
-
-            // fit height
-            float height = (font.getFontDescriptor().getCapHeight()
-                    + -font.getFontDescriptor().getDescent()) * font.getFontMatrix().getScaleY();
-            if (height <= 0)
+            else
             {
-                height = font.getBoundingBox().getHeight() * font.getFontMatrix().getScaleY();
+                // calculate a font size which fits at least x lines
+                float fontSize = scaledContentHeight
+                        / (MINIMUM_LINES_TO_FIT_IN_A_MULTILINE_FIELD * lineHeight);
+                // don't return a font size larger than the default
+                return Math.min(fontSize, DEFAULT_FONT_SIZE);
             }
-
-            float heightBasedFontSize = contentRect.getHeight() / height * yScalingFactor;
-
-            return Math.min(heightBasedFontSize, widthBasedFontSize);
         }
-        return fontSize;
+
+        // fit width
+        float width = font.getStringWidth(value) * font.getFontMatrix().getScaleX();
+        float widthBasedFontSize = contentRect.getWidth() / width * xScalingFactor;
+
+        // fit height
+        float height = calculateLineHeight(font, font.getFontMatrix().getScaleY());
+
+        float heightBasedFontSize = contentRect.getHeight() / height * yScalingFactor;
+
+        return Math.min(heightBasedFontSize, widthBasedFontSize);
     }
 
     /**
